@@ -33,7 +33,7 @@ def get_config(runner,
     Args:
         runner (Runner): Runner for the pipeline. Will be provided by RV.
         raw_uri (str): Directory where the raw data resides
-        processed_uri (str): Directory for storing processed data. 
+        processed_uri (str): Directory for storing processed data.
                              E.g. crops for testing.
         root_uri (str): Directory where all the output will be written.
         external_model (bool, optional): If True, use an external model defined
@@ -97,7 +97,11 @@ def get_config(runner,
             channel_order=[0, 1, 2], uris=[raster_uri])
         label_source = ChipClassificationLabelSourceConfig(
             vector_source=GeoJSONVectorSourceConfig(
-                uri=label_uri, default_class_id=1, ignore_crs_field=True),
+                uri=label_uri,
+                ignore_crs_field=True,
+                transformers=[
+                    ClassInferenceTransformerConfig(default_class_id=1)
+                ]),
             ioa_thresh=0.5,
             use_intersection_over_cell=False,
             pick_min_class_id=False,
@@ -174,7 +178,12 @@ def get_config(runner,
         external_loss_def=external_loss_def)
 
     backend = PyTorchChipClassificationConfig(
-        data=data, model=model, solver=solver, test_mode=test)
+        data=data,
+        model=model,
+        solver=solver,
+        test_mode=test,
+        log_tensorboard=True,
+        run_tensorboard=False)
 
     pipeline = ChipClassificationConfig(
         root_uri=root_uri,
