@@ -24,6 +24,8 @@ def get_config(runner,
                processed_uri: str,
                root_uri: str,
                nochip: bool = True,
+               multiband: bool = False,
+               external_model: bool = False,
                test: bool = False) -> ObjectDetectionConfig:
     """Generate the pipeline config for this task. This function will be called
     by RV, with arguments from the command line, when this example is run.
@@ -31,17 +33,22 @@ def get_config(runner,
     Args:
         runner (Runner): Runner for the pipeline. Will be provided by RV.
         raw_uri (str): Directory where the raw data resides
-        processed_uri (str): Directory for storing processed data. 
+        processed_uri (str): Directory for storing processed data.
                              E.g. crops for testing.
         root_uri (str): Directory where all the output will be written.
         nochip (bool, optional): If True, read directly from the TIFF during
             training instead of from pre-generated chips. The analyze and chip
             commands should not be run, if this is set to True. Defaults to
             False.
+        multiband (bool, optional): If True, all 4 channels (R, G, B, & IR)
+            available in the raster source will be used. If False, only
+            IR, R, G (in that order) will be used. Defaults to False.
+        external_model (bool, optional): If True, use an external model defined
+            by the ExternalModuleConfig. Defaults to True.
         test (bool, optional): If True, does the following simplifications:
             (1) Uses only the first 2 scenes
             (2) Uses only a 2000x2000 crop of the scenes
-            (2) Enables test mode in the learner, which makes it use the
+            (3) Enables test mode in the learner, which makes it use the
                 test_batch_sz and test_num_epochs, among other things.
             Defaults to False.
 
@@ -55,11 +62,17 @@ def get_config(runner,
         train_ids = train_ids[:2]
         val_ids = val_ids[:2]
 
+    if multiband:
+        channel_order = [0, 1, 2, 3]
+        channel_display_groups = {'RGB': [0, 1, 2], 'IR': [3]}
+    else:
+        channel_order = [0, 1, 2]
+        channel_display_groups = None
+
     def make_scene(id: str) -> SceneConfig:
-        raster_uri = join(raw_uri,
-                          '4_Ortho_RGBIR/top_potsdam_{}_RGBIR.tif'.format(id))
+        raster_uri = join(raw_uri, f'4_Ortho_RGBIR/top_potsdam_{id}_RGBIR.tif')
         label_uri = join(processed_uri, 'labels', 'all',
-                         'top_potsdam_{}_RGBIR.json'.format(id))
+                         f'top_potsdam_{id}_RGBIR.json')
 
         if test:
             crop_uri = join(processed_uri, 'crops',
@@ -73,17 +86,19 @@ def get_config(runner,
             raster_uri = crop_uri
 
         raster_source = RasterioSourceConfig(
-            uris=[raster_uri], channel_order=[0, 1, 2])
+            uris=[raster_uri], channel_order=channel_order)
 
         vector_source = GeoJSONVectorSourceConfig(
-            uri=label_uri, default_class_id=0, ignore_crs_field=True)
+            uri=label_uri,
+            ignore_crs_field=True,
+            transformers=[ClassInferenceTransformerConfig(default_class_id=0)])
         label_source = ObjectDetectionLabelSourceConfig(
             vector_source=vector_source)
 
         return SceneConfig(
             id=id, raster_source=raster_source, label_source=label_source)
 
-    class_config = ClassConfig(names=['vehicle'])
+    class_config = ClassConfig(names=['vehicle'], colors=['red'])
     scene_dataset = DatasetConfig(
         class_config=class_config,
         train_scenes=[make_scene(id) for id in train_ids],
@@ -93,7 +108,6 @@ def get_config(runner,
     img_sz = chip_sz
 
     chip_options = ObjectDetectionChipOptions(neg_ratio=5.0, ioa_thresh=0.9)
-
     if nochip:
         window_opts = ObjectDetectionGeoDataWindowConfig(
             method=GeoDataWindowMethod.random,
@@ -110,13 +124,60 @@ def get_config(runner,
             scene_dataset=scene_dataset,
             window_opts=window_opts,
             img_sz=img_sz,
-            num_workers=4)
+            num_workers=4,
+            plot_options=PlotOptions(
+                channel_display_groups=channel_display_groups))
     else:
-        data = ObjectDetectionImageDataConfig(img_sz=img_sz, num_workers=4)
+        data = ObjectDetectionImageDataConfig(
+            img_sz=img_sz,
+            num_workers=4,
+            plot_options=PlotOptions(
+                channel_display_groups=channel_display_groups))
+
+    if external_model:
+        """This demonstrates how to use an external model for object detection,
+        but to successfully use this functionality with different settings, the
+        following things should be kept in mind:
+
+          (1) Torchvision does not expose its object detection models via
+            torch hub (https://github.com/pytorch/vision/issues/1945). So, to
+            use those, you will need to fork the torchvision repo and manually
+            add those models or corresponding factory functions to hubconf.py.
+            Example: github.com/AdeelH/vision/blob/det_hubconf_0.10/hubconf.py.
+            Further, you should ensure that the branch of the fork is the same
+            version as the version in Raster Vision's Docker image; or, if
+            using outside Docker, it should match the version of the local
+            torchvision installation.
+          (2) The external model should behave exactly like torchvision
+            object detection models. This includes, but might not be limited
+            to:
+                - Accepting targets as dicts with keys: 'boxes' and 'labels'.
+                - Accepting 1-indexed class labels.
+                - Computing losses internally and returning them in a dict
+                during training.
+                - Returning predictions as dicts with keys: 'boxes', 'labels',
+                and 'scores'.
+        """
+
+        model = ObjectDetectionModelConfig(
+            external_def=ExternalModuleConfig(
+                github_repo='AdeelH/vision:det_hubconf_0.12',
+                name='ssd',
+                entrypoint='ssd300_vgg16',
+                force_reload=True,
+                entrypoint_kwargs={
+                    # torchvision OD models need add an additional null class,
+                    # so +1 is needed here
+                    'num_classes': len(class_config.names) + 1,
+                    'pretrained': False,
+                    'pretrained_backbone': True
+                }))
+    else:
+        model = ObjectDetectionModelConfig(backbone=Backbone.resnet18)
 
     backend = PyTorchObjectDetectionConfig(
         data=data,
-        model=ObjectDetectionModelConfig(backbone=Backbone.resnet18),
+        model=model,
         solver=SolverConfig(
             lr=1e-4,
             num_epochs=10,
