@@ -25,7 +25,7 @@ def get_config(runner,
     Args:
         runner (Runner): Runner for the pipeline. Will be provided by RV.
         raw_uri (str): Directory where the raw data resides
-        processed_uri (str): Directory for storing processed data. 
+        processed_uri (str): Directory for storing processed data.
                              E.g. crops for testing.
         root_uri (str): Directory where all the output will be written.
         nochip (bool, optional): If True, read directly from the TIFF during
@@ -66,7 +66,11 @@ def get_config(runner,
 
         label_source = ObjectDetectionLabelSourceConfig(
             vector_source=GeoJSONVectorSourceConfig(
-                uri=label_uri, default_class_id=0, ignore_crs_field=True))
+                uri=label_uri,
+                ignore_crs_field=True,
+                transformers=[
+                    ClassInferenceTransformerConfig(default_class_id=0)
+                ]))
 
         return SceneConfig(
             id=id, raster_source=raster_source, label_source=label_source)
