@@ -1,6 +1,5 @@
 import csv
 from io import StringIO
-import tempfile
 import os
 
 import rasterio
@@ -8,9 +7,11 @@ from shapely.strtree import STRtree
 from shapely.geometry import shape, mapping
 from shapely.ops import transform
 
+from rastervision.pipeline.file_system import get_tmp_dir
 from rastervision.core import Box
 from rastervision.core.data import (RasterioCRSTransformer,
-                                    GeoJSONVectorSourceConfig)
+                                    GeoJSONVectorSourceConfig,
+                                    ClassInferenceTransformerConfig)
 from rastervision.pipeline.file_system import (file_to_str, file_exists,
                                                get_local_path, upload_or_copy,
                                                make_dir, json_to_file)
@@ -34,17 +35,17 @@ def get_scene_info(csv_uri):
     return list(reader)
 
 
-def crop_image(image_uri, window, crop_uri):
+def crop_image(image_uri: str, window: Box, crop_uri: str):
     im_dataset = rasterio.open(image_uri)
     rasterio_window = window.rasterio_format()
     im = im_dataset.read(window=rasterio_window)
 
-    with tempfile.TemporaryDirectory() as tmp_dir:
+    with get_tmp_dir() as tmp_dir:
         crop_path = get_local_path(crop_uri, tmp_dir)
         make_dir(crop_path, use_dirname=True)
 
         meta = im_dataset.meta
-        meta['width'], meta['height'] = window.get_width(), window.get_height()
+        meta['height'], meta['width'] = window.size
         meta['transform'] = rasterio.windows.transform(rasterio_window,
                                                        im_dataset.transform)
 
@@ -93,7 +94,11 @@ def save_image_crop(image_uri,
                 crs_transformer = RasterioCRSTransformer.from_dataset(
                     im_dataset)
                 geojson_vs_config = GeoJSONVectorSourceConfig(
-                    uri=label_uri, default_class_id=0, ignore_crs_field=True)
+                    uri=label_uri,
+                    ignore_crs_field=True,
+                    transformers=[
+                        ClassInferenceTransformerConfig(default_class_id=0)
+                    ])
                 vs = geojson_vs_config.build(class_config, crs_transformer)
                 geojson = vs.get_geojson()
                 geoms = []
