@@ -1,12 +1,18 @@
-from os.path import join
+from typing import TYPE_CHECKING, Dict, Iterator, Optional
+from os.path import join, basename
 import uuid
 
-from rastervision.pipeline.file_system import (make_dir, json_to_file)
+from rastervision.pipeline.file_system import json_to_file
 from rastervision.core.data.label import ObjectDetectionLabels
-from rastervision.core.utils.misc import save_img
-from rastervision.core.data_sample import DataSample
 from rastervision.pytorch_backend.pytorch_learner_backend import (
     PyTorchLearnerSampleWriter, PyTorchLearnerBackend)
+from rastervision.pytorch_learner.dataset import (
+    ObjectDetectionSlidingWindowGeoDataset)
+
+if TYPE_CHECKING:
+    import numpy as np
+    from rastervision.core.data import Scene
+    from rastervision.core.data_sample import DataSample
 
 
 class PyTorchObjectDetectionSampleWriter(PyTorchLearnerSampleWriter):
@@ -50,25 +56,26 @@ class PyTorchObjectDetectionSampleWriter(PyTorchLearnerSampleWriter):
 
         super().__exit__(type, value, traceback)
 
-    def write_sample(self, sample: DataSample):
+    def write_sample(self, sample: 'DataSample'):
         """
         This writes a training or validation sample to
         (train|valid)/img/{scene_id}-{ind}.png and updates
         some COCO data structures.
         """
-        split = 'train' if sample.is_train else 'valid'
-        split_dir = join(self.sample_dir, split)
-        img_dir = join(split_dir, 'img')
-        make_dir(img_dir)
-        img_fn = '{}-{}.png'.format(sample.scene_id, self.sample_ind)
-        img_path = join(img_dir, img_fn)
-        save_img(sample.chip, img_path)
+        split_name = 'train' if sample.is_train else 'valid'
 
-        images = self.splits[split]['images']
-        annotations = self.splits[split]['annotations']
+        img_path = self.get_image_path(split_name, sample)
+        self.write_chip(sample.chip, img_path)
+        self.update_coco_data(split_name, sample, img_path)
+        self.sample_ind += 1
+
+    def update_coco_data(self, split_name: str, sample: 'DataSample',
+                         img_path: str):
+        images = self.splits[split_name]['images']
+        annotations = self.splits[split_name]['annotations']
 
         images.append({
-            'file_name': img_fn,
+            'file_name': basename(img_path),
             'id': self.sample_ind,
             'height': sample.chip.shape[0],
             'width': sample.chip.shape[1]
@@ -87,38 +94,40 @@ class PyTorchObjectDetectionSampleWriter(PyTorchLearnerSampleWriter):
                 'category_id': int(class_id)
             })
 
-        self.sample_ind += 1
-
 
 class PyTorchObjectDetection(PyTorchLearnerBackend):
     def get_sample_writer(self):
-        output_uri = join(self.pipeline_cfg.chip_uri, '{}.zip'.format(
-            str(uuid.uuid4())))
+        output_uri = join(self.pipeline_cfg.chip_uri, f'{uuid.uuid4()}.zip')
         return PyTorchObjectDetectionSampleWriter(
             output_uri, self.pipeline_cfg.dataset.class_config, self.tmp_dir)
 
-    def predict(self, scene, chips, windows):
-        """Return predictions for a chip using model.
+    def predict_scene(self,
+                      scene: 'Scene',
+                      chip_sz: int,
+                      stride: Optional[int] = None) -> ObjectDetectionLabels:
+        if stride is None:
+            stride = chip_sz
 
-        Args:
-            chips: [[height, width, channels], ...] numpy array of chips
-            windows: List of boxes that are the windows aligned with the chips.
-
-        Return:
-            Labels object containing predictions
-        """
         if self.learner is None:
             self.load_model()
 
-        batch_out = self.learner.numpy_predict(chips, raw_out=False)
-        labels = ObjectDetectionLabels.make_empty()
+        # Important to use self.learner.cfg.data instead of
+        # self.learner_cfg.data because of the updates
+        # Learner.from_model_bundle() makes to the custom transforms.
+        base_tf, _ = self.learner.cfg.data.get_data_transforms()
+        ds = ObjectDetectionSlidingWindowGeoDataset(
+            scene, size=chip_sz, stride=stride, transform=base_tf)
 
-        for chip_ind, out in enumerate(batch_out):
-            window = windows[chip_ind]
-            boxes = out['boxes']
-            class_ids = out['class_ids']
-            scores = out['scores']
-            boxes = ObjectDetectionLabels.local_to_global(boxes, window)
-            labels += ObjectDetectionLabels(boxes, class_ids, scores=scores)
+        predictions: Iterator[Dict[str, 'np.ndarray']] = (
+            self.learner.predict_dataset(
+                ds,
+                raw_out=True,
+                numpy_out=True,
+                progress_bar=True,
+                progress_bar_kw=dict(desc=f'Making predictions on {scene.id}'))
+        )
+
+        labels = ObjectDetectionLabels.from_predictions(
+            ds.windows, predictions)
 
         return labels
